# coding: utf8
""" A collection of useful helpers """

__version__ = "1.7.1"

## NOTE: no modules imported here should import `decimal` (otherwise
##   `use_cdecimal` might become problematic for them)

from .base import *

# import warnings


## Put the other primary modules in the main module namespace
## ... but do not fail
## Conclusion: this is generally a bad idea.
# from . import runlib
# try:
#     from . import lzmah, lzcat
# except ImportError as e:
#     warnings.warn("Unable to import lzma helpers: %r" % (e,))
# try:
#     from . import psql
# except ImportError as e:
#     warnings.warn("Unable to import psql helpers: %r" % (e,))
# try:
#     from . import twisted_aux
# except ImportError as e:
#     warnings.warn("Unable to import twisted helpers: %r" % (e,))



# coding: utf8
""" madstuff: other stuff stuff """

import re
import urlparse
from pyaux import dotdict


__all__ = (
    'Url',
    '_url_re',
    '_cut', 'IPNBDFDisplay',
)


class Url(dotdict):
    """ urlparse.ParseResult and parse_qs[l] in a dict-like non-lazy form """
    _components = (
        'scheme', 'netloc', 'path', 'params', 'query', 'fragment',
        # from ResultMixin
        'username', 'password', 'hostname', 'port',
    )  # <scheme>://<netloc>/<path>;<params>?<query>#<fragment>

    # TODO: urlunescaped parts

    def __init__(self, url, **kwa):
        self.url = url
        urldata = urlparse.urlparse(url, **kwa)
        for key in self._components:
            val = getattr(urldata, key)
            setattr(self, key, val)

        self.query_str = urldata.query
        self.queryl = urlparse.parse_qs(urldata.query)
        self.query = dict(urlparse.parse_qsl(urldata.query))
        # TODO?: self.query = pyaux.dicts.MVOD(urldata.query)


_url_re = (
    ur'''(?i)\b((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+'''
    ur'''[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+'''
    ur'''(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'".,<>?«»“”‘’]))''')


def _cut(s, l):
    if len(s) <= l:
        return s
    return s[:l] + u'…'


def IPNBDFDisplay(df, *ar, **kwa):
    """ A helper to display a pandas DataFrame in the IPython notebook.

    Recommended things to do:
    pandas.options.display.max_colwidth = 2000
    H = IPNBDFDisplay

    :param head: (default 200) runs df = df.head(head); has default to
        avoid accidentally outputing too much; needs explicit
        `head=None` to disable.
    """
    from IPython.display import display, HTML
    kwa.setdefault('float_format', lambda v: '%.6f' % (v,))
    head = kwa.pop('head', 200)
    cutlinks = kwa.pop('cutlinks', True)
    do_display = kwa.pop('display', True)

    if head:
        df = df.head(head)

    html = df.to_html(*ar, **kwa)

    if cutlinks:
        cutlinks = 80 if cutlinks is True else cutlinks

        def cutlink(link):
            return '<a href="%s">%s</a>' % (link, _cut(link, cutlinks))
        html = re.sub(
            _url_re,
            lambda match: cutlink(match.group(0)),
            html)

    # TODO?: option to insert copious '<wb/>'s in all cells

    if do_display:
        display(HTML(html))
    else:
        return HTML(html)



#!/usr/bin/env python
# coding: utf8

import json
import sys


def main():
    indent = int(os.environ.get('INDENT') or '2')
    data_in = sys.stdin.read()

    try:
        data = json.loads(data_in)
    except ValueError as exc:
        sys.stderr.write("#  -!!---- %s\n" % (exc,))
        sys.stdout.write(data_in)
        return 13
    data_out = json.dumps(
        data, indent=indent, sort_keys=True, ensure_ascii=False)
    sys.stdout.write(data_out)
    sys.stdout.write("\n")


if __name__ == '__main__':
    sys.exit(main())



# coding: utf8
"""
Various dict-related special classes.

>>> mod = remvdodd([(1, 1), (2, 1.4), (1, 2)])
>>> mod
remvdodd(1: 1, 2: 1.4, 1: 2)
>>> mod['mod'] = mod
>>> mod
remvdodd(1: 1, 2: 1.4, 1: 2, 'mod': ...)
>>> mod.modmod = mod['mod']
>>> mod
remvdodd(1: 1, 2: 1.4, 1: 2, 'mod': ..., 'modmod': ...)
>>> import copy
>>> mc = copy.deepcopy(mod)
>>> mod.mod is mod.mod, mc.mod is not mod.mod
(True, True)
>>> mcc = mod.copy()
>>> mcc.mod is mod.mod
True
>>> mcc.clear()
>>> mcc
remvdodd()
>>> mod
remvdodd(1: 1, 2: 1.4, 1: 2, 'mod': ..., 'modmod': ...)
>>> mod.mod is mod
True
>>> mod.update(u1='y')
>>> mod
remvdodd(1: 1, 2: 1.4, 1: 2, 'mod': ..., 'modmod': ..., 'u1': 'y')
>>> mod.update_inplace([(2, 2.6), (3, 3.6)])
>>> mod
remvdodd(1: 1, 2: 2.6, 1: 2, 'mod': ..., 'modmod': ..., 'u1': 'y', 3: 3.6)
>>> mod.pop(3)
3.6
>>> mod.update_replace([(1, -1)])
>>> mod
remvdodd(2: 2.6, 'mod': ..., 'modmod': ..., 'u1': 'y', 1: -1)
>>> del mod['modmod']
>>> mod
remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1)
>>> mod._data
((2, 2.6), ('mod', remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1)), ('u1', 'y'), (1, -1))
>>> mod._data is not mod.items(), mod._data == tuple(mod.items())
(True, True)
>>> mod[1] = 3
>>> mod
remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1, 1: 3)
>>> mod.update(u1='n')
>>> mod
remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1, 1: 3, 'u1': 'n')
>>> mod
remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1, 1: 3, 'u1': 'n')
>>> mod.lists()
[(2, [2.6]), ('mod', [remvdodd(2: 2.6, 'mod': ..., 'u1': 'y', 1: -1, 1: 3, 'u1': 'n')]), ('u1', ['y', 'n']), (1, [-1, 3])]
>>> mod.popitem()
('u1', 'n')
>>> mod.popitem(last=False)
(2, 2.6)
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: -1, 1: 3)
>>> mod.deduplicate()
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3)
>>> mod.u1 = 'yy'
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3, 'u1': 'yy')
>>> mod.deduplicate(how='first')
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3)
>>> list(reversed(mod))
[1, 'u1', 'mod']
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3)
>>> mod.setdefault(5, 2)
2
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3, 5: 2)
>>> mod.setdefault(5, 1)
2
>>> mod
remvdodd('mod': ..., 'u1': 'y', 1: 3, 5: 2)
"""
# not tested:  [iter](keys|values|lists), pop, fromkeys, eq
# ipy to doctest:  sed -r 's/^In[^:]+: />>> /; s/^Out[^:]+: //; /^ *$/ d'
# test to locals:  from pyaux import dicts; reload(dicts); from pyaux.dicts import *; exec '\n'.join(v[4:] for v in dicts.__doc__.split('\n\n')[-1].splitlines() if v.startswith('>>> '))

import copy
import itertools
from UserDict import DictMixin

from pyaux.base import uniq_g
from pyaux.base import dotdict


__all__ = (
    'ODReprMixin', 'OrderedDict',
    'MVOD',
    'hasattr_x',
    'dotdictx', 'defaultdictx', 'DefaultDotDictMixin',
    'dodd', 'mvdodd', 'remvdodd', 'redodd',
)


class ODReprMixin(object):
    """ A mixin for ordered dicts that provides two different representations
    and a wrapper for handling self-referencing structures. """

    def __irepr__(self):
        """ The usual (default) representation of an ordereddict """
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self.items())

    def __drepr__(self):
        """ A slightly more visual-oriented representation of an ordereddict """
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%s)' % (
            self.__class__.__name__,
            ', '.join(
                '%r: %r' % (key, val)
                for key, val in self.iteritems()))

    def __repr__(self, _repr_running={}, fn=__drepr__):
        """ Wrapped around __drepr__ that makes it possible to
        represent infinitely-recursive dictionaries of this type. """
        # NOTE: version variety; might be a _get_ident or something else.
        from thread import get_ident as _get_ident

        call_key = (id(self), _get_ident())
        if call_key in _repr_running:
            # TODO?: make a YAML-like naming & referencing?
            # (too complicated for a repr() though)
            return '...'
        _repr_running[call_key] = 1
        try:
            return fn(self)
        finally:
            del _repr_running[call_key]


# https://pypi.python.org/pypi/ordereddict
# or /usr/lib/python2.7/collections.py
# with modifications
class OrderedDict(dict, ODReprMixin, DictMixin):

    __end = None
    __map = None

    def __init__(self, *args, **kwds):
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        # try:
        #     self.__end
        # except AttributeError:
        #     self.clear()
        if self.__end is None:
            # XX: What was that, an inheritance support?
            # (the original OrderedDict code even does an AttributeError catch)
            self.clear()
        # self.clear()
        self.update(*args, **kwds)

    def clear(self):
        self.__end = end = []
        # sentinel node for doubly linked list
        end += [None, end, end]
        # key --> [key, prev, next]
        self.__map = {}
        dict.clear(self)

    def __setitem__(self, key, value):
        if key not in self:
            end = self.__end
            curr = end[1]
            curr[2] = end[1] = self.__map[key] = [key, curr, end]
        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        key, prev, next = self.__map.pop(key)
        prev[2] = next
        next[1] = prev

    def __iter__(self):
        end = self.__end
        curr = end[2]
        while curr is not end:
            yield curr[0]
            curr = curr[2]

    def __reversed__(self):
        end = self.__end
        curr = end[1]
        while curr is not end:
            yield curr[0]
            curr = curr[1]

    def popitem(self, last=True):
        if not self:
            raise KeyError('dictionary is empty')
        if last:
            key = reversed(self).next()
        else:
            key = iter(self).next()
        value = self.pop(key)
        return key, value

    def __reduce__(self):
        items = [[k, self[k]] for k in self]
        tmp = self.__map, self.__end
        del self.__map, self.__end
        inst_dict = vars(self).copy()
        self.__map, self.__end = tmp
        if inst_dict:
            return (self.__class__, (items,), inst_dict)
        return self.__class__, (items,)

    def keys(self):
        return list(self)

    setdefault = DictMixin.setdefault
    update = DictMixin.update
    pop = DictMixin.pop
    values = DictMixin.values
    items = DictMixin.items
    iterkeys = DictMixin.iterkeys
    itervalues = DictMixin.itervalues
    iteritems = DictMixin.iteritems

    def copy(self):
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, iterable, value=None):
        d = cls()
        for key in iterable:
            d[key] = value
        return d

    def __eq__(self, other):
        if isinstance(other, OrderedDict):
            if len(self) != len(other):
                return False
            for p, q in zip(self.items(), other.items()):
                if p != q:
                    return False
            return True
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self == other


# ######
# Other stuff
# ######


class MVOD(ODReprMixin, dict):
    """ MultiValuedOrderedDict: A not-very-optimized (most write operations
    are at least O(N) with the re-hashing cost) somewhat-trivial verison.
    Stores a tuple of pairs as the actual data (in `_data`), uses it for
    iteration, caches dict(data) as self for optimized key-access. """
    # TODO?: support unhashable keys (by skipping them in the cache)
    # TODO?: make the setitem behaviour configurable per instance

    __data = ()

    def __init__(self, *args, **kwds):
        self.update(*args, **kwds)

    def clear(self):
        self._data_checked = ()  # still does the _update_cache (over property)

    def _update_cache(self):
        dict.clear(self)
        dict.update(self, self.__data)

    def _preprocess_data(self, data):
        """ Make sure the passed data is a list of pairs; returns a tuple with
        the pair-tuples. """
        res = []
        for i, item in enumerate(data):
            lv = len(item)  ## Paranoidally avoid calling it twice.
            if lv != 2:
                raise ValueError((
                    "dictionary update sequence element #%d has"
                    " length %r; 2 is required") % (i, lv))
            key, val = item
            res.append((key, val))
        return tuple(res)

    def _process_upddata(self, args, kwds, preprocess=True):
        """ Convert some function call args into list of key-value pairs (same
        as `dict(*args, **kwds)` does).
        Returns a tuple with pair-tuples.  """
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        arg = args[0] if args else []
        if isinstance(arg, MVOD):  ## support init / update from antother MVOD
            arg = arg._data
        elif isinstance(arg, dict):
            arg = arg.iteritems()
        if kwds:
            # Append the keywords to the other stuff
            arg = itertools.chain(arg, kwds.iteritems())
            # raise TypeError('initializing an ordered dict from keywords is not recommended')
        if preprocess:
            arg = self._preprocess_data(arg)  ## NOTE: iterates over it and makes a tuple.
        return arg

    def update_append(self, *args, **kwds):
        """ ...
        (the usual equivalent of dict.update)

        WARNING: appends the data; thus, multiple `update`s will cause
        it to grow; thus, might need to be `.deduplicate`d.
        """
        data_new = self._process_upddata(args, kwds)
        # XXXXX: A dilemma:
        #  * If self._data is a list, then it can be mutated by the
        #    user without updating the cache
        #  * If self._data is a tuple, then it has to be re-created for
        #    any change (current version).
        #  * If self._data is a custom proxy to a list... dunno. TODO?
        #    (similar to the django.utils.datastructures.ImmutableList
        #    except not tuple-derived)
        self._data_checked = self._data + data_new

    def update_replace(self, *args, **kwds):
        """ A closer equivalent of dict.update that removes all the previous
        occurrences of the keys that are updated and appends the new ones to
        the end. The new keys still can occur multiple times. """
        data_new = self._process_upddata(args, kwds)
        keys = set(key for key, val in data_new)
        pre_data = tuple((key, val) for key, val in self._data
                         if key not in keys)
        self._data_checked = pre_data + data_new

    def update_inplace(self, *args, **kwds):
        """ A closer equivalent of OrderedDict.update that replaces the
        previous occurrences at the point of first occurrence.  Does not yet
        support multiple items handling in the updated keys. """
        data_new = self._process_upddata(args, kwds)
        news = dict(*args, **kwds)
        # XXXX/TODO: To update with multiple values will have to do `MVOD(*args,
        #   **kwds).lists()`, and (configurably) pop either each item from the
        #   list when it occurs or just one item each time.
        pre_data = tuple((key, news.pop(key, val))
                         for key, val in self._data)
        # Add the non-previously-existing ones.
        data_new = [(key, val) for key, val in data_new if key in news]
        self._data_checked = pre_data + tuple(data_new)

    update = update_append

    def __delitem__(self, key):
        self._data_checked = tuple((k, v) for k, v in self._data if k != key)

    # A set of properties to set the data with or without checking it
    # for validity.
    @property
    def _data(self):
        return self.__data

    @_data.setter
    def _data(self, val):
        self.__data = self._preprocess_data(val)
        self._update_cache()

    @property
    def _data_checked(self):
        return self.__data

    @_data_checked.setter
    def _data_checked(self, val):
        # just a check that can be optimized out.
        assert isinstance(val, tuple)
        self.__data = val
        self._update_cache()

    # def __getitem__:  inherited from `dict`

    def __setitem__(self, key, value):
        return self.update(((key, value),))
        # self._data_checked = self._data + ((key, value),)

    def deduplicate(self, how='last'):
        """ ...
        NOTE: this method is equivalent to deduplicate_last by default. """
        if how == 'first':
            pass
        elif how == 'last':
            return self.deduplicate_last()
        else:
            raise ValueError("Unknown deduplication `how`: %r" % (how,))
        self._data_checked = tuple(uniq_g(self._data, key=lambda item: item[0]))

    def deduplicate_last(self):
        data_pre = uniq_g(reversed(self._data), key=lambda item: item[0])
        data_pre = list(data_pre)
        self._data_checked = tuple(reversed(data_pre))

    def deduplicated(self, **kwa):
        cp = self.copy()
        cp.deduplicate(**kwa)
        return cp

    def __copy__(self):
        return self.__class__(self._data)

    def __deepcopy__(self, memo=None):
        if memo is None:
            memo = {}
        result = self.__class__()
        memo[id(self)] = result
        result._data_checked = copy.deepcopy(self.__data, memo)
        return result

    copy = __copy__  # XXX: re-check.

    # XXX/TODO: PY3.  (see django.utils.datastructures.MultiValueDict)

    def iteritems(self):
        # XXXX: no dict-changed-while-iterating handling.
        return iter(self._data)

    def iterlists(self):
        """ MultiValueDict-like (django) method. Not very optimal. """
        # XXX: no dict-changed-while-iterating handling.
        order = []
        lists = {}
        for key, val in self.iteritems():
            if key not in lists:
                lst = lists[key] = [val]
                order.append((key, lst))
            else:
                lists[key].append(val)
        for key, lst in order:
            yield key, lst  # lists[k]

    def iterkeys(self):
        for key, _ in self.iteritems():
            yield key

    def __iter__(self):
        return self.iterkeys()

    def __reversed__(self):
        for key, val in reversed(self._data):
            yield key

    # Copypaste from UserDict.DictMixin
    def itervalues(self):
        for _, val in self.iteritems():
            yield val

    def values(self):
        return [val for _, val in self.iteritems()]

    def items(self):
        return list(self.iteritems())

    def keys(self):
        return [k for k, _ in self.iteritems()]

    def lists(self, **kwa):
        return list(self.iterlists(**kwa))

    # Bit more copypaste from UserDict.DictMixin (because most other
    # methods from it are unnecessary)
    def pop(self, key, *args):
        if len(args) > 1:
            raise TypeError("pop expected at most 2 arguments, got %r" % (1 + len(args),))
        try:
            value = self[key]
        except KeyError:
            if args:
                return args[0]
            raise
        del self[key]
        return value

    def setdefault(self, key, failobj=None):
        if key not in self:
            self[key] = failobj
        return self[key]

    def popitem(self, last=True):
        if not self:
            raise KeyError('dictionary is empty')
        data = self._data
        if last:
            item = data[-1]
            self._data_checked = data[:-1]
        else:
            item = data[0]
            self._data_checked = data[1:]
        return item

    @classmethod
    def fromkeys(cls, iterable, value=None):
        return cls([(k, value) for k in iterable])

    def __eq__(self, other):
        """ ...

        WARN: `mvod == od` and `od == mvod` might have different
        results (because OD doesn't handle MVODs).
        """
        if isinstance(other, (MVOD, OrderedDict)):
            # In the end it comes down to items.
            return dict.__eq__(self, other) and (self.items() == other.items())
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self == other

    # __nonzero__ does not need overriding as dict handles that.

    # XXX: some other methods?


# NOTE: A simpler form is available in the `base`
class dotdictx(dict):
    """ A dict subclass with items also available over
    attributes. Skips the attributes starting with '_' (for
    mixinability) """

    def __getattr__(self, name):
        if name.startswith('__'):  # NOTE: two underscores.
            return super(dotdictx, self).__getattr__(name)
        try:
            return self[name]
        except KeyError as e:
            raise AttributeError(e)

    def __setattr__(self, name, value):
        if name.startswith('_'):
            return super(dotdictx, self).__setattr__(name, value)
        self[name] = value


class defaultdictx(dict):
    """ A purepython & simplified `defaultdict` version that doesn't change
    the interface except for the `_default` attribute (with the
    default_factory) """
    _default = None

    def __getitem__(self, name):
        try:
            return super(defaultdictx, self).__getitem__(name)
        except KeyError:
            if self._default is not None:
                val = self._default()
                self[name] = val
                return val
            raise


def hasattr_x(obj, name):
    """ A safer `hasattr` that only checks for simple attributes
    rather than properties or __getattr__ results. """
    try:
        object.__getattribute__(obj, name)
    except AttributeError:
        return False
    return True


class DefaultDotDictMixin(dotdict, defaultdictx):
    """ A class that tries to combine DefaultDict and dotdict without causing
    too much of a mess. NOTE: skips _attributes on setattr and __attributes on
    getattr. """

    def __getattr__(self, name):
        # Mostly necessary to avoid `defaultdict`ing some special
        # methods like __getstate__ that weren't defined on the class.
        # (could disable the defaultdict'ing for that, though)
        if name.startswith('__'):  # NOTE: two underscores.
            return self.__getattribute__(name)  # Basically `raise AttributeError`.
        return super(DefaultDotDictMixin, self).__getattr__(name)  # __getitem__

    def __setattr__(self, name, value):
        # if name in self.__dict__:
        # if hasattr_x(self, name):
        #     self.__dict__[name] = value
        #     return

        # Mostly necessary for class code that sets attributes like
        # '_OrderedDict__end' (or the defaultdictx._default)
        if name.startswith('_'):
            # Basically `object.__setattr__(…)`
            # WARN: querying `d._attr` still sets it to the
            # default. Not necessarily problematic though.
            return dict.__setattr__(self, name, value)

        return super(DefaultDotDictMixin, self).__setattr__(name, value)  # __setitem__


class dodd(DefaultDotDictMixin, OrderedDict):
    """ DotOrderedDefaultDict. Set `_default` attribute on it to a factory to
    use it as a defaultdict. NOTE: ignores attributes starting with '_' """
    pass


class mvdodd(DefaultDotDictMixin, MVOD):
    pass


class redodd(dodd):
    """ Recursive dodd (by default) """
redodd._default = redodd


class remvdodd(mvdodd):
    """ ... """
remvdodd._default = remvdodd


# TODO: test... things...

# TODO?: Add some other sort of multivaluedness
# (django.utils.datastructures.MultiValueDict).  (and maybe add a
# separate class compatible with id and/or the django.http.QueryDict
# or something)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
""" json -> yaml for pretty-reading. """

import sys
import json
import yaml
import argparse
from pyaux.base import colorize_yaml as colorize


def cmd_make_parser(**kwa):
    parser = argparse.ArgumentParser(
        description=(
            "json -> yaml for pretty-reading"
        )
    )
    parser.add_argument(
        '-f', '--no-default-flow-style',
        dest='default_flow_style', action='store_false',
        default=None,
        help="pyyaml's `default_flow_style=False` (generally longer)",
    )
    parser.add_argument(
        '-x', '--default-flow-style',
        dest='default_flow_style', action='store_true',
        default=None,
        help="pyyaml's `default_flow_style=True` (more json-like)",
    )
    parser.add_argument(
        '-c', '--color',
        choices=('yes', 'always', 'no', 'never', 'auto'),
        default='auto',
        nargs='?',
        help=("Colorize the output (requires pygments) (default: 'auto')"
              " (`ls --color` semantics)"),
    )
    # parser.add_argument(
    #     '-c', dest='color', action='store_true',
    #     help="Colorize the output (requires pygments) (same as `--color=yes`)",
    # )
    parser.add_argument(
        '-nu',
        dest='allow_unicode', action='store_false',
        default=True,
        help="Do not allow unicode in output (pyyaml's 'allow_unicode')",
    )

    return parser


def main():

    parser = cmd_make_parser()
    params = parser.parse_args()

    ## TODO?: support input file
    data = sys.stdin.read()
    data_data = json.loads(data)

    kwa = dict(
        default_flow_style=params.default_flow_style,
        allow_unicode=params.allow_unicode,
    )

    out = yaml.safe_dump(data_data, **kwa)

    ## Result:
    ##   `--color=no` always skips this
    ##   `--color`, `--color=yes`, `-c` always do this
    ##   ``, `--color=auto` make this isatty-conditional.
    if ((params.color == 'auto' and sys.stdout.isatty())
            ## NOTE: `'auto'` is default, `None` means it was
            ## specified without an argument (equals to 'always')
            or params.color in ('yes', 'always', True, None)):

        ## pygments doesn't like utf-8 as-is; make it unicode
        if isinstance(out, bytes):
            out = out.decode('utf-8')

        ## TODO?: support --color=auto
        out = colorize(out)

    ## Apply the default encoding and don't even allow to change it
    if isinstance(out, unicode):
        out = out.encode('utf-8')

    ## TODO?: support output file
    sys.stdout.write(out)
    if out[-1] != '\n':  ## Just in case
        sys.stdout.write('\n')
    sys.stdout.flush()


if __name__ == '__main__':
    sys.exit(main())



# coding: utf8
""" madstuff: datadiff stuff """

import yaml
import difflib
import itertools
from pyaux.base import colorize_yaml, colorize_diff


__all__ = (
    '_dumprepr',
    '_diff_pre_diff', '_diff_datadiff_data', 'datadiff', 'p_datadiff',
)


def _dumprepr(val, no_anchors=True, colorize=False, try_ujson=True, **kwa):
    """ Advanced-ish representation of an object (using YAML) """
    dumper = yaml.SafeDumper

    # NOTE: this means it'll except on infinitely-recursive data.
    if no_anchors:
        dumper = type(
            'NoAliasesSafeDumper', (dumper,),
            dict(ignore_aliases=lambda self, data: True))

    params = dict(default_flow_style=False, allow_unicode=True,
                  Dumper=dumper)
    params.update(kwa.get('yaml_kwa', {}))

    def maybe_colorize(text):
        if not colorize:
            return text
        return colorize_yaml(text, **kwa)

    res = ''
    try:
        res += maybe_colorize(yaml.dump(val, **params))
    except Exception:
        if not try_ujson:
            raise
        # ujson can handle many objects somewhat-successfully. But can
        # segfault while doing that.
        import ujson
        res += "# Unable to serialize directly!\n"
        prepared_value = ujson.loads(ujson.dumps(val))
        res += maybe_colorize(yaml.dump(prepared_value, **params))

    return res


def _diff_pre_diff(val, **kwa):
    """ Prepare a value for diff-ing """
    _repr = kwa.get('_repr', _dumprepr)
    res = _repr(val, **kwa)
    res = res.splitlines()
    return res


def _diff_datadiff_data(val1, val2, **kwa):
    """ Do the diff and return the data """
    val1_p = _diff_pre_diff(val1, **kwa)
    val2_p = _diff_pre_diff(val2, **kwa)
    res = difflib.unified_diff(val1_p, val2_p)
    return res


def datadiff(val1, val2, colorize=False, colorize_yaml=False, **kwa):
    """ Return a values diff string """
    kwa['colorize'] = colorize_yaml  # NOTE: controversial
    data = _diff_datadiff_data(val1, val2, **kwa)

    # line_limit
    _ll = kwa.pop('line_limit', 200)
    if _ll:
        data_base = data
        data = list(itertools.islice(data_base, _ll))
        try:
            next(data_base)
        except StopIteration:
            pass
        else:
            data.append(u'...')  # u'…'

    res = '\n'.join(data)
    if colorize:
        res = colorize_diff(res, **kwa)
    return res


def p_datadiff(val1, val2, **kwa):
    """ Print the values diff """
    # TODO: yaml coloring *and* diff coloring?
    print datadiff(val1, val2, **kwa)



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

version = '1.7.1'  # should be the same as pyaux.__version__

LONG_DESCRIPTION = """
Collection of helpers and useful things for Python

"""

try:
    LONG_DESCRIPTION = (
        LONG_DESCRIPTION +
        open(os.path.join(os.path.dirname(__file__), 'README.rst')).read())
except Exception as _exc:
    print "Pkg-description error:", _exc


setup_kwargs = dict(
    name='pyaux',
    version=version,
    description='pyaux',  ## XX
    long_description=LONG_DESCRIPTION,
    # classifiers=[],
    # keywords='...,...',
    author='HoverHell',
    author_email='hoverhell@gmail.com',
    url='https://github.com/HoverHell/pyaux',
    download_url='https://github.com/HoverHell/pyaux/tarball/%s' % (version,),
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'lzcat.py = pyaux.lzcat:_lzcat_main',
            'lzma.py = pyaux.lzmah:_lzma_main',
            'fjson_yaml = pyaux.bin.fjson_yaml:main',
            'fjson.py = pyaux.bin.fjson:main',
            'fmsgp_json = pyaux.bin.fmsgp_json:main',
        ],
    },
    install_requires=['six'],
    extras_require={
        # Things that are useful to simply have around:
        'recommended': [
            'ipython', 'ipdb', 'PyYAML',
            'atomicfile', 'cdecimal',
            # 'requests', 'pycurl',
        ],
        # All things that are known to be used in some part of this
        # library or another.
        'known': [
            'django',  # in the psql helper
            'Twisted',  # bunch of twisted stuff here
            'Cython',  # at least one pyx module
            'pandas',  # here and there
            'Pygments',  # json / yaml coloring
            'pylzma',  # helpers for it
            'simplejson',  # optional, for speed
            # 'pp',  # so special I won't even include it here
            'line_profiler',
            # 'pyzmq',  # also too rare
        ],
    },
    dependency_links=[
        # 'https://github.com/sashka/atomicfile/tarball/master#egg=atomicfile',  # on pypi now
    ],
    # package_data={},
    # include_package_data=True,
    # zip_safe=False,
)


if __name__ == '__main__':
    setup(**setup_kwargs)



# coding: utf8
""" Things that are not quite okay to use in most cases.

Also, things that are useful in an ipython interactice shell.
"""

from .madness_datadiff import *
from .madness_oneliny import *
from .madness_reprstuff import *
from .madness_stuffstuff import *


# __all__ includes everything from submodules too
__all__ = (
    # repr stuff
    'GenReprWrap', 'GenReprWrapWrap',
    # the oneliners and debug-useful stuff
    '_try', '_try2', '_iter_ar', '_filter',
    '_filter_n', '_print', '_ipdbg', '_uprint',
    '_yprint', 'p_o_repr',
    # diff stuff
    '_dumprepr',
    '_diff_pre_diff', '_diff_datadiff_data', 'datadiff', 'p_datadiff',
    # stuff stuff
    'Url',
    '_url_re',
    '_cut', 'IPNBDFDisplay',
    # __builtin__ hacks
    '_olt_into_builtin',
    '_into_builtin',
)


# # Builtin-madness # #


def _into_builtin(d):
    """ Helper to put stuff (like the one-liner-helpers) into builtins """
    import __builtin__
    for k, v in d.items():
        setattr(__builtin__, k, v)


# For _into_builtin
__all_stuff = locals()
__all_stuff_e = dict((k, globals().get(k)) for k in __all__)


try:
    # better pprint
    from IPython.lib.pretty import pprint, pretty
    __all_stuff.update(pprint=pprint, pretty=pretty, pformat=pretty)
    __all_stuff_e.update(pprint=pprint, pretty=pretty, pformat=pretty)
except ImportError as __e:
    print "What, no IPython?", __e


# For explicit call:
def _olt_into_builtin():
    return _into_builtin(__all_stuff_e)


# For use as execfile()
if locals().get('__into_builtin'):
    _olt_into_builtin()


# Recommendation: put
#     from pyaux import madness
#     madness._olt_into_builtin()
# in the
#     ~/.config/ipython/profile_default/ipython_config.py



# coding: utf8
# NOTE: no modules imported here should import `decimal` (otherwise
#   `use_cdecimal` might become problematic for them)

import os
import sys

from copy import deepcopy
import errno
import functools
import itertools
from itertools import chain, repeat, islice
import json
import logging
import math
import re
import six
import time
import traceback
import unicodedata

from pyaux import ranges
from pyaux.ranges import *

from pyaux import interpolate
from pyaux.interpolate import *

__all__ = (
    'bubble',
    'window',
    'dotdict',
    'SmartDict',
    'DebugPlug', 'repr_call',
    'dict_fget',
    'dict_fsetdefault',
    'split_list',
    'use_cdecimal',
    'use_exc_ipdb',
    'use_exc_log',
    'use_colorer',
    'obj2dict',
    'mk_logging_property',
    'sign',
    'try_parse',
    'human_sort_key',
    'reversed_blocks',
    'reversed_lines',
    'lazystr',
    'list_uniq',
    'o_repr',
    'chunks',
    'chunks_g',
    # 'runlib',
    # 'lzmah',
    # 'lzcat',
    # 'psql',
    'to_bytes',
    'to_unicode',
) + ranges.__all__ + interpolate.__all__


def bubble(*args, **kwargs):
    """ Prettified super():
    Calls `super(ThisClass, this_instance).this_method(...)`.

    Not super-performant but quite prettifying ("Performance is 5
    times worse than super() call").

    src:
    http://stackoverflow.com/questions/2706623/super-in-python-2-x-without-args/2706703#2706703
    """
    import inspect

    def find_class_by_code_object(back_self, method_name, code):
        for cls in inspect.getmro(type(back_self)):
            if method_name in cls.__dict__:
                method_fun = getattr(cls, method_name)
                if method_fun.im_func.func_code is code:
                    return cls

    frame = inspect.currentframe().f_back
    back_self = frame.f_locals['self']
    method_name = frame.f_code.co_name

    for _ in xrange(5):
        code = frame.f_code
        cls = find_class_by_code_object(back_self, method_name, code)
        if cls:
            super_ = super(cls, back_self)
            return getattr(super_, method_name)(*args, **kwargs)
        try:
            frame = frame.f_back
        except Exception:
            return


# Iterate over a 'window' of adjacent elements
# http://stackoverflow.com/questions/6998245/iterate-over-a-window-of-adjacent-elements-in-python
def window(seq, size=2, fill=0, fill_left=False, fill_right=False):
    """ Returns a sliding window (of width n) over data from the iterable:
      s -> (s0,s1,...s[n-1]), (s1,s2,...,sn), ...
    """
    ssize = size - 1
    it = chain(
        repeat(fill, ssize * fill_left),
        iter(seq),
        repeat(fill, ssize * fill_right))
    result = tuple(islice(it, size))
    if len(result) == size:  # `<=` if okay to return seq if len(seq) < size
        yield result
    for elem in it:
        result = result[1:] + (elem,)
        yield result


class dotdict(dict):
    """ A simple dict subclass with items also available over attributes """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            # A little less confusing way:
            _et, _ev, _tb = sys.exc_info()
            six.reraise(AttributeError, _ev, _tb)

    def __setattr__(self, name, value):
        self[name] = value


# Compat alias
SmartDict = dotdict


_dotdictify_marker = object()


class dotdictify(dict):
    """ Recursive automatic doctdict thingy """

    def __init__(self, value=None):
        if value is None:
            pass
        elif isinstance(value, dict):
            for key in value:
                self.__setitem__(key, value[key])
        else:
            raise TypeError('expected a dict')

    def __setitem__(self, key, value):
        if isinstance(value, dict) and not isinstance(value, dotdictify):
            value = dotdictify(value)
        dict.__setitem__(self, key, value)

    def __getitem__(self, key):
        found = self.get(key, _dotdictify_marker)
        if found is _dotdictify_marker:
            found = dotdictify()
            dict.__setitem__(self, key, found)
        return found

    __setattr__ = __setitem__
    __getattr__ = __getitem__


def repr_call(ar, kwa):
    """ A helper function for pretty-printing a function call arguments """
    r = ', '.join("%r" % (v,) for v in ar)
    if kwa:
        if r:
            r += ', '
        r + ', '.join('%s=%r' % (k, v) for k, v in kwa.iteritems())
    return r


dbgs = {}   # global object for easier later access of dumped `__call__`s


def DebugPlug(name, mklogger=None):
    """ Create and return a recursive duck-object for plugging in
    place of other objects for debug purposes.

    :param name: name for tracking the object and its (child) attributes.

    :param mklogger: a function of `name` (str) that returns a new callable(msg:
      str) used for logging.  See code for an example.
    """
    # The construction with mkClass is for removing the need of
    #   `__getattr__`ing the name and logger.
    import logging

    def mklogger_default(name):
        logger = logging.getLogger(name)
        return logger.debug

    if mklogger is None:
        mklogger = mklogger_default

    log = mklogger(name)

    class DebugPlugInternal(object):
        """ An actual internal class of the DebugPlug """

        def __call__(self, *ar, **kwa):
            log("called with (%s)" % repr_call(ar, kwa))
            global dbgs  ## Just to note it
            dbgs.setdefault(name, {})
            dbgs[name]['__call__'] = (ar, kwa)

        def __getattr__(self, attname):
            namef = "%s.%s" % (name, attname)
            # Recursive!
            dpchild = DebugPlug(name=namef, mklogger=mklogger)
            # setattr(self, attname, dpchild)
            object.__setattr__(self, attname, dpchild)
            return dpchild

        def __setattr__(self, attname, value):
            log("setattr: %s = %r" % (attname, value))
            global dbgs
            dbgs.setdefault(name, {})
            dbgs[name][attname] = value
            return object.__setattr__(self, attname, value)

    return DebugPlugInternal()


# ###### dict-lazies

def dict_fget(D, k, d):
    """ dict_get(D, k, d) -> D[k] if k in D, else d().
      - a lazy-evaluated dict.get.
      (d is mandatory but can be None). """
    if k in D:
        return D[k]
    return d() if d is not None else d


def dict_fsetdefault(D, k, d):
    """ dict_fsetdefault(D, k, d) -> dict_fget(D, k, d), also set D[k]=d() if k not in D.
      - a lazy-evaluated dict.setdefault.
      (d is mandatory but can be None).  """
    # Can be `D[k] = dict_fget(D, k, d); return D[k]`, but let's micro-optimize.
    # NOTE: not going over 'keyerror' for the defaultdict or alike classes.
    if k in D:
        return D[k]
    v = d() if d is not None else d
    D[k] = v
    return v


def split_list(lst, cond):
    """ Split list items into two into (matching, non_matching) by
      `cond(item)` callable """
    res1, res2 = [], []
    for i in lst:
        if cond(i):
            res1.append(i)
        else:
            res2.append(i)
    return res1, res2


# ###### Monkey-patching of various things:

def use_cdecimal():
    """ Do a hack-in replacement of `decimal` with `cdecimal`.
    Should be done before importing other modules.

    Also see
    http://adamj.eu/tech/2015/06/06/swapping-decimal-for-cdecimal-on-python-2/
    for a possibly more reliable way.
    """
    import decimal  # maybe not needed
    import cdecimal
    sys.modules['decimal'] = cdecimal


def use_exc_ipdb():
    """ Set unhandled exception handler to automatically start ipdb """
    import pyaux.exc_ipdb as exc_ipdb
    exc_ipdb.init()


def use_exc_log():
    """ Set unhandled exception handler to verbosely log the exception """
    import pyaux.exc_log as exc_log
    exc_log.init()


def use_colorer():
    """ Wrap logging's StreamHandler.emit to add colors to the logged
      messages based on log level """
    # TODO: make a ColorerHandlerMixin version
    import pyaux.Colorer as Colorer
    Colorer.init()


# ...

def obj2dict(o, add_type=False, add_instance=False, do_lists=True,
             dict_class=dotdict):
    """" Recursive o -> o.__dict__ """
    kwa = dict(add_type=add_type, add_instance=add_instance,
               do_lists=do_lists, dict_class=dict_class)

    if hasattr(o, '__dict__'):
        res = dict_class()
        for k, v in o.__dict__.iteritems():
            res[k] = obj2dict(v, **kwa)
        if add_type:
            res['__class__'] = o.__class__
        if add_instance:
            res['__instance__'] = o
        return res
    # Recurse through other types too:
    # NOTE: There might be subclasses of these that would not be
    # processed here.
    elif isinstance(o, dict):
        return dict_class((k, obj2dict(v, **kwa)) for k, v in o.iteritems())
    elif isinstance(o, list):
        return [obj2dict(v, **kwa) for v in o]

    return o  # something else - return as-is.


def mk_logging_property(actual_name, logger_name='_log'):
    """ Creates a property that logs the value and the caller in the
    setter, using logger under `self`'s logger_name, and stores the value
    under actual_name on `self` """

    def do_get(self):
        return getattr(self, actual_name)

    def do_set(self, val):
        tb = traceback.extract_stack(limit=2)[0]
        # # or:
        # next((r.f_code.co_filename, r.f_lineno, r.f_code.co_name) for r in (sys._getframe(1),))
        # # that is,
        # r = sys._getframe(1)
        # co = r.f_code
        # co.co_filename, r.f_lineno, co.co_name
        setattr(self, actual_name, val)
        getattr(self, logger_name).debug(
            "%s set to %r from %s:%d, in %s",
            actual_name, val, tb[0], tb[1], tb[2])

    return property(do_get, do_set)


def sign(v):
    """ Sign of value: `return cmp(v, 0)` """
    return cmp(v, 0)


# ######  "Human" sorting, advanced
# partially from quodlibet/quodlibet/util/__init__.py
# partially from comix/src/filehandler.py


def try_parse(v, fn=int):
    """ 'try parse' (with fn) """
    try:
        return fn(v)
    except Exception, e:
        return v


# Note: not localized (i.e. always as dot for decimal separator)
_re_alphanum_f = re.compile(r'[0-9]+(?:\.[0-9]+)?|[^0-9]+')


def _split_numeric_f(s):
    return [try_parse(v, fn=float) for v in _re_alphanum_f.findall(s)]


# Or to avoid interpreting numbers as float:
_re_alphanum_int = re.compile(r'\d+|\D+')


def _split_numeric(s):
    return [try_parse(v, fn=int) for v in _re_alphanum_int.findall(s)]


# Primary function:
def human_sort_key(s, normalize=unicodedata.normalize, floats=True):
    """ Sorting key for 'human' sorting """
    if not isinstance(s, unicode):
        s = s.decode("utf-8")
    s = normalize("NFD", s.lower())
    split_fn = _split_numeric_f if floats else _split_numeric
    return s and split_fn(s)


# ###### Reading files backwards
# http://stackoverflow.com/a/260433/62821

def reversed_blocks(fileobj, blocksize=4096):
    """ Generate blocks of file's contents in reverse order.  """
    fileobj.seek(0, os.SEEK_END)
    here = fileobj.tell()
    while 0 < here:
        delta = min(blocksize, here)
        fileobj.seek(here - delta, os.SEEK_SET)
        yield fileobj.read(delta)
        here -= delta


def reversed_lines(fileobj):
    """ Generate the lines of file in reverse order.  """
    tail = []           # Tail of the line whose head is not yet read.
    for block in reversed_blocks(fileobj):
        # A line is a list of strings to avoid quadratic concatenation.
        # (And trying to avoid 1-element lists would complicate the code.)
        linelists = [[line] for line in block.splitlines()]
        linelists[-1].extend(tail)
        for linelist in reversed(linelists[1:]):
            yield ''.join(linelist)
        tail = linelists[0]
    if tail:
        yield ''.join(tail)


# ####### ...

class ThrottledCall(object):
    """ Decorator for throttling calls to some functions (e.g. logging).
    Defined as class for various custom attributes and methods.
    Attributes:
      `handle_skip`: function(self, *ar, **kwa) to call when a call is
        skipped. (default: return None)
    Methods: `call_something`
    """

    # _last_call_time = None
    _call_time_throttle = None
    _call_cnt = 0  # (kept accurate; but can become ineffectively large)
    _call_cnt_throttle = 0  # next _call_cnt to call at
    _call_val = object()  # (some unique value at start)

    def __init__(self, fn=None, sec_limit=None, cnt_limit=None):
        """ `fn`: function to call (can be customized later).
        `sec_limit`: skip call if less than `sec_limit` seconds since the
          last call
        `cnt_limit`: call only once each `cnt_limit` calls
        """
        self.fn = fn
        # # mimickry, v2
        # self.__call__ = wraps(fn)(self.__call__)
        self.sec_limit = sec_limit
        self.cnt_limit = cnt_limit
        doc = "%s (throttled)" % (fn.__doc__,)
        self.__call__.__func__.__doc__ = self.__doc__ = doc
        self.handle_skip = lambda self, *ar, **kwa: None

    def __call__(self, *ar, **kwa):
        return self.call_something(self.fn, *ar, **kwa)

    def call_something(self, fn, *ar, **kwa):
        """ Call some (other) function with the same throttling """
        now = time.time()
        # do_call = True
        # # NOTE: `throttle_cnt(throttle_sec(fn))` is emulated if both
        # # are set.
        if self.cnt_limit is not None:
            self._call_cnt += 1
            if self._call_cnt >= self._call_cnt_throttle:
                self._call_cnt_throttle += self.cnt_limit
            else:
                return self.handle_skip(self, *ar, **kwa)
        if self.sec_limit is not None:
            if now > self._call_time_throttle:
                self._call_time_throttle = now + self.sec_limit
            else:
                return self.handle_skip(self, *ar, **kwa)
        res = fn(*ar, **kwa)
        return res

    def call_value(self, val, fn, *ar, **kwa):
        """ Call if the value hasn't changed (applying the other
        throttling parameters as well). Contains undocumented feature
        """
        if self._call_val != val:
            self._call_val = val
            if fn is None:
                fn = self.fn
            return self.call_something(fn, *ar, **kwa)

    def __repr__(self):
        return "<throttled_call(%r)>" % (self.fn,)

    # # Optional: mimickry
    # def __repr__(self):
    #     return repr(self.fn)
    # def __getattr__(self, v):
    #     return getattr(self.fn, v)


@functools.wraps(ThrottledCall)
def throttled_call(fn=None, *ar, **kwa):
    """ Wraps the supplied function with ThrottledCall (or generates a
    wrapper with the supplied parameters). """
    if fn is not None:
        if callable(fn):
            # mimickry, v3
            return functools.wraps(fn)(ThrottledCall(fn, *ar, **kwa))
        else:  # supplied some arguments as positional?
            # XX: make a warning?
            ar = (fn,) + ar
    return lambda fn: functools.wraps(fn)(ThrottledCall(fn, *ar, **kwa))


class lazystr(object):
    """ A simple class for lazy-computed processing into string,
      e.g. for use in logging.
    Example: `log(13, "stuff is %r",
      lazystr(lambda: ', '.join(stuff)))`
    Note: no caching.
    """

    def __init__(self, fn):
        self.fn = fn

    def __str__(self):
        return str(self.fn())

    def __repr__(self):
        return repr(self.fn())


def uniq_g(lst, key=lambda v: v):
    """ Get unique elements of an iterable preserving its order and optionally
    determining uniqueness by hash of a key """
    known = set()
    for v in lst:
        k = key(v)
        if k not in known:
            yield v
            known.add(k)
    # ...


def uniq(lst, key=lambda v: v):
    """ RTFS """
    return list(uniq_g(lst, key=key))


list_uniq = uniq_g


# Helper for o_repr that displays '???'
class ReprObj(object):
    """ A class for inserting specific text in __repr__ outputs.  """

    def __init__(self, txt):
        self.txt = txt

    def __repr__(self):
        return self.txt


# _err_obj = type('ErrObj', (object,), dict(__repr__=lambda self: '???'))()
_err_obj = ReprObj('???')


# It is similar to using self.__dict__ in repr() but works over dir()
def o_repr_g(
        o, _colors=False, _colors256=False, _colorvs=None, _method=2,
        _private=False, _callable=False):
    """ Represent (most of) data on a python object in readable
    way. Useful default for a __repr__.
    WARN: does not handle recursive structures; use carefully.  """

    # TODO: handle recursive structures (similar to dict.__repr__)
    # TODO: process base types (dict / list / ...) in a special way

    def _color(x16, x256=None):
        if not _colors:
            return ''
        if x16 is None and x256 is None:
            return '\x1b[00m'
        if _colors256 and x256:
            return '\x1b[38;5;' + str(x256) + 'm'
        return '\x1b[0' + str(x16) + 'm'

    _colorvs_add = _colorvs
    _colorvs = dict(
        base=('1;37', '230'),  # Base (punctuation)  # white / pink-white
        clsn=('1;36', '123'),  # Class name  # bright cyan / cyan-white
        attr=('0;32', '120'),  # Attribute name  # dark-green / green-white
        func=('0;34', '75'),  # Function name  # blue / blue-cyan
        val=('0;37', '252'),  # Value data  # light-gray / light-light-gray
        clear=(None, None),
    )
    _colorvs.update(_colorvs_add or {})

    def _colorv(n):
        return _color(*_colorvs[n])

    yield _colorv("base")
    yield '<'
    yield _colorv("clsn")
    yield str(o.__class__.__name__)
    yield _colorv("base")
    yield '('

    if _method == 3:
        # V3: check type for properties
        o_type = type(o)

    first = True

    for n in sorted(dir(o)):

        # skip 'private' stuff
        if n.startswith('_') and not _private:
            continue

        if first:
            first = False
        else:
            yield _colorv("base")
            yield ', '

        keytype = "attr"
        has_val = True

        # V2: try but fail
        if _method == 2:
            try:
                v = getattr(o, n)
                if callable(v):  # skip functions (... and other callables)
                    if not _callable:
                        has_val = False
                    keytype = "func"
            except Exception:
                v = _err_obj

        if _method == 3:
            # V3: check type for properties
            v_m = getattr(o_type, n, None)
            if v_m is not None and isinstance(v_m, property):
                continue  # skip properties
            v = getattr(o, n)

            # skip functions (... and other callables)
            if callable(v):
                if not _callable:
                    has_val = False
                keytype = "func"

        yield _colorv(keytype)
        yield str(n)  # NOTE: some cases (e.g. functions) will remain just names

        if has_val:
            try:
                v = repr(v)
            except Exception:
                v = _err_obj
            yield _colorv("base")
            yield '='
            yield _colorv("val")
            yield v

    yield _colorv("base")
    yield ')>'
    yield _colorv("clear")


def o_repr(o, **kwa):
    return ''.join(o_repr_g(o, **kwa))


class OReprMixin(object):
    def __repr__(self):
        return o_repr(self)


def stdin_lines(strip_newlines=True):
    """ Iterate over stdin lines in a 'line-buffered' way """
    while True:
        try:
            l = sys.stdin.readline()
        except KeyboardInterrupt:
            return
        if not l:
            break
        # This might not be the case if the stream terminates with a non-newline at the end.
        if strip_newlines and l[-1] == '\n':
            l = l[:-1]
        yield l


def stdout_lines(gen):
    """ Send lines from a generator / iterable to stdout in a line-buffered way. """
    for l in gen:
        sys.stdout.write("%s\n" % (l,))
        sys.stdout.flush()


def dict_merge(target, source, instancecheck=None, dictclass=dict,
               del_obj=object(), _copy=True, inplace=False):
    """ do update() on 'dict of dicts of di...' structure recursively.
    Also, see sources for details.
    NOTE: does not keep target's specific tree structure (forces source's)
    :param del_obj: allows for deletion of keys if the key in the `source` is set to this.

    >>> data = {}
    >>> data = dict_merge(data, {'open_folders': {'my_folder_a': False}})
    >>> data
    {'open_folders': {'my_folder_a': False}}
    >>> data = dict_merge(data, {'open_folders': {'my_folder_b': True}})
    >>> data
    {'open_folders': {'my_folder_b': True, 'my_folder_a': False}}
    >>> _del = object()
    >>> data = dict_merge(data, {'open_folders': {'my_folder_b': _del}}, del_obj=_del)
    >>> data
    {'open_folders': {'my_folder_a': False}}
    """
    if instancecheck is None:  # funhorrible ducktypings
        # instancecheck = lambda iv: isinstance(iv, dict)

        def instancecheck_default(iv):
            return hasattr(iv, 'iteritems')

        instancecheck = instancecheck_default

    # Recursive parameters shorthand
    kwa = dict(instancecheck=instancecheck, dictclass=dictclass, del_obj=del_obj)

    if _copy or not inplace:  # 'either is default'
        target = deepcopy(target)

    for k, v in source.iteritems():
        if v is del_obj:
            target.pop(k, None)
        elif instancecheck(v):  # (v -> source -> iteritems())
            # NOTE: if target[k] wasn't a dict - it will be, now.
            target[k] = dict_merge(
                dict_fget(target, k, dictclass), v, **kwa)
        else:  # nowhere to recurse into - just replace
            # NOTE: if target[k] was a dict - it won't be, anymore.
            target[k] = v

    return target


def _sqrt(var):
    """ Duck-compatible sqrt(), needed to support classes like Decimal.

    Approximately the same as in the python3's `statistics`. """
    try:
        return var.sqrt()
    except AttributeError:
        return math.sqrt(var)


class IterStat(object):
    """ Iterative single-pass computing of mean and variance.

    Error is on the rate of 1e-08 for 1e6 values in the range of
    0..1e6, both for mean and for stddev. """
    # http://www.johndcook.com/standard_deviation.html

    def __init__(self, vals=None, start=0):
        self.start = start
        self.old_mean = None
        self.mean = self.stdx = start
        self.cnt = 0

        if vals:
            for val in vals:
                self.send(val)

    def send(self, val):
        self.cnt += 1
        if self.cnt == 1:
            self.mean = val
        else:
            self.mean = self.mean + (val - self.mean) / float(self.cnt)
            self.stdx = self.stdx + (val - self.old_mean) * (val - self.mean)
        self.old_mean = self.mean

    @property
    def variance(self):
        if self.cnt <= 1:
            return self.start
        return self.stdx / (self.cnt)

    @property
    def std(self):
        return _sqrt(self.variance)


def chunks(lst, size):
    """ Yield successive chunks from lst. No padding.  """
    for idx in xrange(0, len(lst), size):
        yield lst[idx:idx + size]


def chunks_g(iterable, size):
    """ Same as 'chunks' but works on any iterable.

    Converts the chunks to tuples for simplicity.
    """
    # http://stackoverflow.com/a/8991553
    it = iter(iterable)
    if size <= 0:
        yield it
        return
    while True:
        chunk = tuple(islice(it, size))
        if not chunk:
            return
        yield chunk


def group(lst, cls=dict):
    """ RTFS.

    Similar to dict(MultiValueDict(lst).lists())

    >>> group([(1, 1), (2, 2), (1, 3)])
    {1: [1, 3], 2: [2]}
    """
    res = cls()
    for key, val in lst:
        try:
            group_list = res[key]
        except KeyError:
            res[key] = [val]
        else:
            group_list.append(val)
    return res


def group2(lst, key=lambda v: v[0]):
    """ RTFS.

    Not particularly better than `group((key(val), val) for val in lst)`.
    """
    res = {}
    for v in lst:
        res.setdefault(key(v), []).append(v)
    return res.items()


def mangle_dict(input_dict, include=None, exclude=None, add=None, _return_list=False):
    """ Functional-style dict editing """
    include = set(include) if include is not None else None
    exclude = set(exclude) if exclude is not None else None
    if include:
        assert not exclude

    items = input_dict.iteritems()
    if include is not None:
        res = [(key, val) for key, val in items
               if key in include]
    elif exclude is not None:
        res = [(key, val) for key, val in items
               if key not in exclude]
    else:
        res = list(items)

    # ... functional-style `update`.
    if add is not None:
        if isinstance(add, dict):
            add = add.iteritems()
        res.extend(add)
    if _return_list:
        return res
    return dict(res)


filterdict = mangle_dict


def colorize(text, fmt, outfmt='term', lexer_kwa=None, formatter_kwa=None, **kwa):
    """ Convenience method for running pygments """
    from pygments import highlight, lexers, formatters
    _colorize_lexers = dict(
        yaml='YamlLexer', diff='DiffLexer',
        py='PythonLexer', py2='PythonLexer', py3='Python3Lexer',
    )
    _colorize_formatters = dict(
        term='TerminalFormatter', term256='Terminal256Formatter',
        html='HtmlFormatter',
    )
    fmt = _colorize_lexers.get(fmt, fmt)
    outfmt = _colorize_formatters.get(outfmt, outfmt)

    lexer_cls = getattr(lexers, fmt)
    formatter_cls = getattr(formatters, outfmt)
    return highlight(
        text, lexer_cls(**(lexer_kwa or {})),
        formatter_cls(**(formatter_kwa or {})))


def colorize_yaml(text, **kwa):
    """ Attempt to colorize the yaml text using pygments (for console
    output) """
    return colorize(text, 'yaml', **kwa)


def colorize_diff(text, **kwa):
    """ Attempt to colorize the [unified] diff text using pygments
    (for console output) """
    return colorize(text, 'diff', **kwa)


def _dict_hash_1(dct):
    """ Simple non-recursive dict -> hash """
    return hash(tuple(sorted(dct.items())))

_dict_hash = _dict_hash_1


# TODO?: some clear-all-memos method
class memoize(object):

    def __init__(self, fn, timelimit=None):  # TODO?: time limit
        self.log = logging.getLogger("%s.%r" % (__name__, self))
        self.fn = fn
        self.mem = {}
        self.timelimit = timelimit
        self.skip_first_arg = False
        functools.update_wrapper(self, fn)

    def __call__(self, *ar, **kwa):
        now = time.time()  # NOTE: before the call
        override = kwa.pop('_memoize_force_new', False)
        # TODO?: cleanup obsolete keys here sometimes.
        try:
            if self.skip_first_arg:
                key = (ar[1:], _dict_hash(kwa))
            else:
                key = (ar, _dict_hash(kwa))
            # XXX/TODO: make `key` a weakref
            then, res = self.mem[key]
        except KeyError:
            pass
        except TypeError:  # e.g. unhashable args
            self.log.warn("memoize: Trying to memoize unhashable args %r, %r", ar, kwa)
            return self.fn(*ar, **kwa)
        else:
            if not override and (self.timelimit is None or (now - then) < self.timelimit):
                # Still okay
                return res
        # KeyError or obsolete result
        res = self.fn(*ar, **kwa)
        self.mem[key] = (now, res)
        return res


def memoize_method(*ar, **cfg):
    """ `memoize` for a method, saving the cache on an instance
    attribute.

    :param memo_attr: name of the attribute to save the cache on.
    :param timelimit: see `memoize`.
    """

    def memoize_method_configured(func):
        memo_attr = cfg.pop('memo_attr', None)
        if memo_attr is None:
            memo_attr = '_cached_%s' % (func.__name__,)

        @functools.wraps(func)
        def memoized_method(self, *ar, **kwa):
            cache = getattr(self, memo_attr, None)
            if cache is None:
                cache = memoize(func, **cfg)
                cache.skip_first_arg = True
                setattr(self, memo_attr, cache)

            return cache(self, *ar, **kwa)

        return memoized_method

    if ar and callable(ar[0]):
        return memoize_method_configured(ar[0])
    return memoize_method_configured


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def _dict_to_hashable_json(dct, dumps=json.dumps):
    """ ...

    NOTE: Not meant to be performant; tends towards collisions.

    Use `id(dct)` for a performant way with the reverse tendency.
    """
    return dumps(dct, skipkeys=True, default=id)


def group3(items, to_hashable=_dict_to_hashable_json):
    """ Same as `group` but supports dicts as keys (and returns list
    of pairs) """
    annotated = [(to_hashable(key), key, val) for key, val in items]
    hashes = dict((keyhash, key) for keyhash, key, val in annotated)
    groups = group((keyhash, val) for keyhash, key, val in annotated).items()
    return [(hashes[keyhash], lst) for keyhash, lst in groups]


def to_bytes(st, default=(lambda val: val), **kwa):
    if isinstance(st, bytes):
        return st
    if not isinstance(st, unicode):
        return default(st)  # normally the value itself
    kwa.setdefault('encoding', 'utf-8')
    return st.encode(**kwa)


def to_unicode(st, default=(lambda val: val), **kwa):
    if isinstance(st, unicode):
        return st
    if not isinstance(st, bytes):
        return default(st)
    kwa.setdefault('encoding', 'utf-8')
    return st.decode(**kwa)


def import_module(name, package=None):
    """ ...

    django.utils.importlib.import_module without the package support
    """
    __import__(name)
    return sys.modules[name]


def import_func(func_path, _check_callable=True):
    """ Get an object (e.g. a function / callable) by import path.

    supports '<module>:<path>' notation as well as '<module>.<func_name>'.
    """
    # # Somewhat borrowed from django.core.handlers.base.BaseHandler.load_middleware
    # from django.utils.importlib import import_module

    _exc_cls = Exception

    if ':' in func_path:
        f_module, f_name = func_path.split(':', 1)
    else:
        try:
            f_module, f_name = func_path.rsplit('.', 1)
        except ValueError as exc:
            raise _exc_cls("func_path isn't a path", func_path, exc)

    f_name_parts = f_name.split('.')

    try:
        mod = import_module(f_module)
    except ImportError as exc:
        raise _exc_cls("func_path's module cannot be imported", func_path, exc)

    try:
        here = mod
        for f_name_part in f_name_parts:
            if not f_name_part:
                continue  ## allows for weirder things to be done.
            here = getattr(here, f_name_part)
        func = here
    except AttributeError as exc:
        raise _exc_cls("func_path's module does not have the specified func", func_path, exc)

    if _check_callable and not callable(func):
        raise _exc_cls("func does not seem to be a callable", func_path, func)

    return func


def next_or_fdefault(it, default=lambda: None, skip_empty=False):
    """
    `next(it, default_value)` with laziness.

    >>> next_or_fdefault([1], lambda: 1/0)
    1
    >>> next_or_fdefault([], lambda: list(range(2)))
    [0, 1]
    """
    if skip_empty:
        it = (val for val in it if val)
    else:
        it = iter(it)
    try:
        return next(it)
    except StopIteration:
        return default()


def iterator_is_over(it, ret_value=False):
    """ Try to consume an item from an iterable `it` and return False
    if it succeeded (the item stays consumed) """
    try:
        val = next(it)
    except StopIteration:
        if ret_value:
            return True, None
        return True
    else:
        if ret_value:
            return False, val
        return False


def dict_is_subset(
        smaller_dict, larger_dict,
        recurse_iterables=False, structure_match=True):
    """ Recursive check "smaller_dict's keys are subset of
    larger_dict's keys.

    NOTE: in practice, supports non-dict values at top.
    """
    kwa = dict(
        recurse_iterables=recurse_iterables,
        structure_match=structure_match,
    )
    if isinstance(smaller_dict, dict):
        if not isinstance(larger_dict, dict):
            if structure_match:
                return False
            return True
        # Both are dicts.
        for key, val in smaller_dict.items():
            try:
                lval = larger_dict[key]
            except KeyError:
                return False
            # 'compare' the values whatever they are
            if not dict_is_subset(val, lval, **kwa):
                return False
    elif recurse_iterables and hasattr(smaller_dict, '__iter__'):
        if not hasattr(larger_dict, '__iter__'):
            if structure_match:
                return False
            return True
        # smaller_value_iter, larger_value_iter
        svi = iter(smaller_dict)
        lvi = iter(larger_dict)
        for sval, lval in itertools.izip(svi, lvi):
            if not dict_is_subset(sval, lval, **kwa):
                return False
        if structure_match:
            if not iterator_is_over(svi) or not iterator_is_over(lvi):
                # One of the iterables was longer and thus was not
                # consumed entirely by the izip
                return False

    return True


def find_files(
        in_dir, fname_re=None, older_than=None, skip_last=None,
        _prewalk=True):
    """ Return all full file paths under the directory `in_dir` whose
    *filenames* match the `fname_re` regexp (if not None) """
    now = time.time()
    walk = os.walk(in_dir)
    if _prewalk:
        walk = list(walk)
    for dir_name, dir_list, file_list in walk:
        if fname_re is not None:
            file_list = (v for v in file_list if re.match(fname_re, v))
        # Annotate with full path:
        file_list = ((os.path.join(dir_name, file_name), file_name)
                     for file_name in file_list)
        if older_than is not None:
            file_list = ((fpath, fname) for fpath, fname in file_list
                         if now - os.path.getmtime(fpath) >= older_than)

        # Convenience shortcut
        if skip_last:
            file_list = sorted(list(file_list), key=lambda v: v[1])
            file_list = file_list[:-skip_last]

        for fpath, fname in file_list:
            yield fpath


if __name__ == '__main__':
    import doctest
    doctest.testmod()
